<?php
declare(strict_types=1);

namespace Catequil\ORM;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Application;

/**
 * Class ConfigProvider
 *
 * @package Catequil\ORM
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'dependencies' => $this->getDependencies(),
            'filters' => $this->getFilters(),
            'orm' => $this->getOrmConfig(),
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                EntityManagerInterface::class => EntityManager::class,
            ],
            'delegators' => [
                Application::class => [
                    Container\CliApplicationDelegatorFactory::class,
                ],
            ],
            'factories' => [
                EntityManager::class => Container\EntityManagerFactory::class,
            ],
        ];
    }

    public function getFilters(): array
    {
        return [
            'factories' => [
                Filter\EntityFilter::class => Filter\EntityFilterFactory::class,
            ],
        ];
    }

    public function getOrmConfig(): array
    {
        return [
            'connection' => 'db.default',
            'proxy_dir' => 'data/orm/proxy',
        ];
    }
}
