<?php
declare(strict_types=1);

namespace Catequil\ORM\Container;

use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Tools\Console\ConsoleRunner;
use Doctrine\ORM\Tools\Console\EntityManagerProvider;
use Laminas\ServiceManager\Factory\DelegatorFactoryInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;
use Symfony\Component\Console\Application;

/**
 * Class CliApplicationDelegatorFactory
 *
 * @package Catequil\ORM\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class CliApplicationDelegatorFactory implements DelegatorFactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param $name
     * @param callable $callback
     * @param array|null $options
     *
     * @return Application
     *
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $name, callable $callback, ?array $options = null): Application
    {
        /** @var Application $cli */
        $cli = $callback();
        $em = $container->get(EntityManagerInterface::class);

        ConsoleRunner::addCommands($cli, new class($em) implements EntityManagerProvider
        {
            private EntityManagerInterface $em;

            public function __construct(EntityManagerInterface $em)
            {
                $this->em = $em;
            }

            public function getDefaultManager(): EntityManagerInterface
            {
                return $this->em;
            }

            public function getManager(string $name): EntityManagerInterface
            {
                return $this->getDefaultManager();
            }
        });

        return $cli;
    }
}