<?php
declare(strict_types=1);

namespace Catequil\ORM\Filter;

use Doctrine\ORM\EntityManagerInterface;
use Laminas\Filter\AbstractFilter;
use RuntimeException;
use Traversable;
use function class_exists;

/**
 * Class EntityFilter
 *
 * @package Catequil\ORM\Filter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-orm
 */
final class EntityFilter extends AbstractFilter
{
    private EntityManagerInterface $em;
    private string $entityClass = '';

    /**
     * @param EntityManagerInterface $em
     * @param array|Traversable|null $options
     */
    public function __construct(EntityManagerInterface $em, array|Traversable|null $options = null)
    {
        $this->em = $em;

        if ($options) {
            $this->setOptions($options);
        }
    }

    /**
     * @param string $entityClass
     */
    public function setEntityClass(string $entityClass): void
    {
        $this->entityClass = $entityClass;
    }

    /**
     * @param int $value
     *
     * @return object
     */
    public function filter($value): object
    {
        if (empty($this->entityClass)) {
            throw new RuntimeException('Entity class not set.');
        }

        if (! class_exists($this->entityClass)) {
            throw new RuntimeException(sprintf(
                "Entity class '%s' not found.",
                $this->entityClass
            ));
        }

        $entity = $this->em->find($this->entityClass, $value);

        if (!$entity) {
            throw new RuntimeException(sprintf(
                "Entity '%s' with id %d not found.",
                $this->entityClass, $value
            ));
        }

        return $entity;
    }
}
